<?php

namespace Magento\Pakkepost\Controller\Adminhtml\Index;

use Magento\Backend\App\Action;
use Magento\Backend\App\Action\Context;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\Controller\Result\JsonFactory;
use Magento\Framework\HTTP\Client\Curl;
use Magento\Sales\Api\OrderRepositoryInterface;
use Psr\Log\LoggerInterface;

class Booking extends Action
{
    protected $jsonResultFactory;
    protected $curl;
    protected $orderRepository;
    protected $scopeConfig;
    protected $logger;

    public function __construct(
        Context $context,
        JsonFactory $jsonResultFactory,
        Curl $curl,
        OrderRepositoryInterface $orderRepository,
        ScopeConfigInterface $scopeConfig,
        LoggerInterface $logger
    ) {
        parent::__construct($context);
        $this->jsonResultFactory = $jsonResultFactory;
        $this->curl = $curl;
        $this->orderRepository = $orderRepository;
        $this->scopeConfig = $scopeConfig;
        $this->logger = $logger;
    }

    /**
     * Main controller method
     * @return \Magento\Framework\Controller\Result\Json
     */
    public function execute()
    {
        // Consolidating configuration retrieval
        $config = $this->_objectManager->get('Magento\Framework\App\Config\ScopeConfigInterface');
        $userId = $config->getValue('pakkepost_config/general/user_id');
        $subscriptionId = $config->getValue('pakkepost_config/general/subscription_id');
        $shopName = $config->getValue('pakkepost_config/general/shop_name');
        $apiMode = $config->getValue('pakkepost_config/general/api_mode');
        $apiUrl = $apiMode === 'live' ? "https://pakkepost.azure-api.net/booking" : "https://pakkepost.azure-api.net/test/booking";
        $senderEmail = $config->getValue('pakkepost_config/general/sender_email');
        $senderPhone = $config->getValue('pakkepost_config/general/sender_phone');
        $senderAddressLine = $config->getValue('pakkepost_config/general/sender_address');
        $senderPostalCode = $config->getValue('pakkepost_config/general/sender_zipcode');
        $senderCity = $config->getValue('pakkepost_config/general/sender_city');

        // Check for required configuration
        if (empty($userId) || empty($subscriptionId) || empty($shopName) || empty($senderEmail) || empty($senderPhone)) {
            $this->logConfigError([$userId, $subscriptionId, $shopName, $apiMode, $senderEmail, $senderPhone]);
            return $this->createErrorResponse('Please check your Pakkepost module configuration; one or more required fields are empty.');
        }

        // Process the request and prepare data
        $requestData = $this->prepareRequestData($userId, $shopName, $senderAddressLine, $senderPostalCode, $senderCity, $senderEmail, $senderPhone);

        // Log the request data
        $this->logRequestData($requestData);

        // Make and process the API call
        return $this->processApiCall($apiUrl, $subscriptionId, $requestData);
    }

    // Additional private methods

    /**
     * Prepare request data for API call
     * @param $userId
     * @param $shopName
     * @param $senderAddressLine
     * @param $senderPostalCode
     * @param $senderCity
     * @param $senderEmail
     * @param $senderPhone
     * @return array
     */
    private function prepareRequestData($userId, $shopName, $senderAddressLine, $senderPostalCode, $senderCity, $senderEmail, $senderPhone)
    {
        $payload = $this->getRequest()->getParam('payload');
        $orderId = $this->getRequest()->getParam('order_id');
        $requestData = [];

        if (!empty($payload)) {
            $payload = json_decode($payload, true);
            if (count($payload) == 1) {
                $order = $this->_objectManager->create('Magento\Sales\Model\Order')->load($payload[0]['order_id']);
                $requestData[] = $this->formatOrderData($order, $userId, $shopName, $senderAddressLine, $senderPostalCode, $senderCity, $senderEmail, $senderPhone, $payload[0]);
            } else {
                foreach ($payload as $orderData) {
                    $order = $this->_objectManager->create('Magento\Sales\Model\Order')->load($orderData['order_id']);
                    // push each order to the request data array
                    $requestData[] = $this->formatOrderData($order, $userId, $shopName, $senderAddressLine, $senderPostalCode, $senderCity, $senderEmail, $senderPhone, $orderData);
                }
            }
        } elseif (!empty($orderId)) {
            $order = $this->_objectManager->create('Magento\Sales\Model\Order')->load($orderId);
            $selectedProduct = $this->getRequest()->getParam('selected_product');
            $productQuantity = $this->getRequest()->getParam('product_quantity');
            $requestData[] = $this->formatOrderData($order, $userId, $shopName, $senderAddressLine, $senderPostalCode, $senderCity, $senderEmail, $senderPhone, ['pkp_product_type' => $selectedProduct, 'pkp_product_count' => $productQuantity]);
        }

        return $requestData;
    }

    /**
     * Format order data for API call
     * @param $order
     * @param $userId
     * @param $shopName
     * @param $senderAddressLine
     * @param $senderPostalCode
     * @param $senderCity
     * @param $senderEmail
     * @param $senderPhone
     * @param $orderData
     * @return array
     */
    private function formatOrderData($order, $userId, $shopName, $senderAddressLine, $senderPostalCode, $senderCity, $senderEmail, $senderPhone, $orderData)
    {
        return [
            "userId" => $userId,
            "senderName" => $shopName,
            "senderAddressLine" => $senderAddressLine,
            "senderPostalCode" => $senderPostalCode,
            "senderCity" => $senderCity,
            "senderReference" => "senderref",
            "senderEmail" => $senderEmail,
            "senderPhoneNumber" => $senderPhone,
            "recipientName" => $order->getShippingAddress()->getFirstname() . ' ' . $order->getShippingAddress()->getLastname(),
            "recipientAddressLine" => $order->getShippingAddress()->getStreetLine(1),
            "recipientAddressLineNumber" => $order->getShippingAddress()->getStreetLine(2),
            "recipientPostalCode" => $order->getShippingAddress()->getPostcode(),
            "recipientCity" => $order->getShippingAddress()->getCity(),
            "recipientReference" => "Recipientref",
            "recipientPhoneNumber" => $order->getShippingAddress()->getTelephone(),
            "recipientEmail" => $order->getShippingAddress()->getEmail(),
            "productId" => $orderData['pkp_product_type'] ?? $this->getRequest()->getParam('selected_product'),
            "additionalServices" => "hjemlevering",
            "numberOfPackages" => $orderData['pkp_product_count'] ?? $this->getRequest()->getParam('product_quantity'),
        ];
    }

    /**
     * Log configuration error
     * @param $configData
     */
    private function logConfigError($configData)
    {
        $this->_objectManager->get('Psr\Log\LoggerInterface')->debug('Pakkepost API request error: ' . PHP_EOL . json_encode($configData));
    }

    /**
     * Create error response
     * @param $errorMessage
     * @return \Magento\Framework\Controller\Result\Json
     */
    private function createErrorResponse($errorMessage)
    {
        $resultJson = $this->jsonResultFactory->create();
        $resultJson->setData([
            'success' => false,
            'error' => $errorMessage,
        ]);
        return $resultJson;
    }

    /**
     * Log request data
     * @param $requestData
     */
    private function logRequestData($requestData)
    {
        $this->_objectManager->get('Psr\Log\LoggerInterface')->debug('Pakkepost API request: ' . PHP_EOL . json_encode([$requestData]));
    }

    /**
     * Make and process API call
     * @param $apiUrl
     * @param $subscriptionId
     * @param $requestData
     * @return \Magento\Framework\Controller\Result\Json
     */
    private function processApiCall($apiUrl, $subscriptionId, $requestData)
    {
        $this->curl->setHeaders([
            'Content-Type' => 'application/json',
            'Ocp-Apim-Subscription-Key' => $subscriptionId,
        ]);
        $this->curl->post($apiUrl, json_encode($requestData));

        $apiResponse = $this->curl->getBody();
        $this->logApiResponse($apiResponse);

        return $this->createApiResult($apiResponse);
    }

    /**
     * Log API response
     * @param $apiResponse
     */
    private function logApiResponse($apiResponse)
    {
        $this->_objectManager->get('Psr\Log\LoggerInterface')->debug('Pakkepost API response: ' . PHP_EOL . $apiResponse);
    }

    /**
     * Create API result
     * @param $apiResponse
     * @return \Magento\Framework\Controller\Result\Json
     */
    private function createApiResult($apiResponse)
    {
        if (filter_var($apiResponse, FILTER_VALIDATE_URL)) {
            $resultJson = $this->jsonResultFactory->create();
            $this->saveOrdersOnSuccess();
            $resultJson->setData([
                'success' => true,
                'url' => $apiResponse,
            ]);
        } else {
            $resultJson = $this->jsonResultFactory->create();
            $resultJson->setData([
                'success' => false,
                'error' => 'API call failed: ' . $apiResponse,
            ]);
        }
        return $resultJson;
    }

    /**
     * Save orders on success
     */
    private function saveOrdersOnSuccess()
    {
        try {
            $payload = $this->getRequest()->getParam('payload');
            $orderId = $this->getRequest()->getParam('order_id');
            if (!empty($payload)) {
                $payload = json_decode($payload, true);

                foreach ($payload as $orderData) {
                    $order = $this->orderRepository->get($orderData['order_id']);
                    $this->updateOrderAttributes($order, $orderData['pkp_product_type'], $orderData['pkp_product_count']);
                }
            } elseif (!empty($orderId)) {
                $order = $this->orderRepository->get($orderId);
                $this->updateOrderAttributes($order, $this->getRequest()->getParam('selected_product'), $this->getRequest()->getParam('product_quantity'));
            }
        } catch (\Exception $e) {
            $this->logger->error('Error updating order: ' . $e->getMessage());
        }
    }

    /**
     * Update order attributes
     * @param $order
     * @param $productType
     * @param $productCount
     */
    private function updateOrderAttributes($order, $productType, $productCount)
    {
        $order->setPakkepostStatus(1);
        $order->setPakkepostProduct($productType);
        $order->setPakkepostProductCount($productCount);
        $this->orderRepository->save($order);
        $this->logger->debug('Pakkepost API order saved: ' . json_encode($order->getData()));
    }

}
